<div class="well">
    <strong>Docs for {{ release }}<br></strong>

    <a id="other">All versions</a>

    <ul id="versionList" style="display: none;">
        <script src="{{ pathto('', 1) }}../dev/_static/docversions.js"></script>
        <script type="text/javascript">
            insert_version_links();
        </script>
    </ul>

 </div>

<script type="text/javascript">
	$("#other").click(function() {
		$("#versionList").toggle();
	});
</script>
